# Generated by Django 2.0.2 on 2018-02-19 08:59

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('automated_logging', '0013_auto_20180218_1106'),
    ]

    operations = [
        migrations.AddField(
            model_name='request',
            name='status',
            field=models.PositiveSmallIntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='field',
            name='model',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='dal_field', to='contenttypes.ContentType'),
        ),
        migrations.DeleteModel(
            name='ModelStorage',
        ),
    ]



"""
Django Automated Logging (DAL) is a package with the purpose of
making logging in django automated and easy.

Simple as that. Really.
"""
default_app_config = 'automated_logging.apps.AutomatedloggingConfig'



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2018-02-16 10:04
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('automated_logging', '0005_auto_20180216_0941'),
    ]

    operations = [
        migrations.AlterField(
            model_name='model',
            name='application',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='atl_model_application', to='automated_logging.Application'),
        ),
    ]



# Generated by Django 2.0.2 on 2018-02-16 15:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('automated_logging', '0010_auto_20180216_1430'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='model',
            options={'verbose_name': 'Changelog', 'verbose_name_plural': 'Changelogs'},
        ),
        migrations.AlterModelOptions(
            name='modelchangelog',
            options={},
        ),
        migrations.AlterModelOptions(
            name='request',
            options={'verbose_name': 'Request', 'verbose_name_plural': 'Requests'},
        ),
        migrations.AlterModelOptions(
            name='unspecified',
            options={'verbose_name': 'Non DJL Message', 'verbose_name_plural': 'Non DJL Messages'},
        ),
    ]



# Generated by Django 2.0.2 on 2018-02-16 14:30

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('automated_logging', '0009_auto_20180216_1006'),
    ]

    operations = [
        migrations.CreateModel(
            name='Field',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=255)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ModelStorage',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=255)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='field',
            name='model',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='automated_logging.ModelStorage'),
        ),
        migrations.AddField(
            model_name='modelobject',
            name='field',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='automated_logging.Field'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2018-02-15 15:40
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('automated_logging', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Application',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=255)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.RenameField(
            model_name='modelobject',
            old_name='application',
            new_name='type',
        ),
        migrations.RemoveField(
            model_name='modelobject',
            name='model',
        ),
        migrations.AlterField(
            model_name='model',
            name='application',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='atl_model_application', to='automated_logging.Application'),
        ),
        migrations.AlterField(
            model_name='request',
            name='application',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='automated_logging.Application'),
        ),
    ]



"""This file handles everything related to saving and deleting model objects."""
import logging
from .. import settings
from django.dispatch import receiver
from django.db.models.signals import m2m_changed
from django.contrib.contenttypes.models import ContentType

from ..models import ModelChangelog, ModelObject, Model, Application
from . import validate_instance, get_current_user


@receiver(m2m_changed, weak=False)
def m2m_callback(sender, instance, action, reverse, model, pk_set, using, **kwargs):
    """
    Detect Many2Many relationship changes and
    append these to existing model or create if needed
    """
    if validate_instance(instance) and settings.AUTOMATED_LOGGING['to_database']:
        if action in ["post_add", 'post_remove']:
            modification = [model.objects.get(pk=x) for x in pk_set]

            if 'al_chl' in instance.__dict__.keys():
                changelog = instance.al_chl
            else:
                changelog = ModelChangelog()

                changelog.information = ModelObject()
                changelog.information.value = repr(instance)
                changelog.information.type = ContentType.objects.get_for_model(instance)
                changelog.information.save()
                changelog.save()

            for f in modification:
                obj = ModelObject()
                obj.value = repr(f)

                try:
                    obj.type = ContentType.objects.get_for_model(f)
                except Exception:
                    logger = logging.getLogger(__name__)
                    logger.debug('Could not determin the type of the modification.')

                obj.save()
                if action == 'post_add':
                    changelog.inserted.add(obj)
                else:
                    changelog.removed.add(obj)

            changelog.save()

            if 'al_evt' in instance.__dict__.keys():
                target = instance.al_evt
            else:
                target = Model()
                target.user = get_current_user()
                target.action = 2 if action == 'post_add' else 2
                target.save()

                ct = ContentType.objects.get_for_model(instance).app_label
                target.application = Application.objects.get_or_create(name=ct)[0]
                target.information = ModelObject()
                target.information.value = repr(instance)
                target.information.type = ContentType.objects.get_for_model(instance)
                target.information.save()

                instance.al_evt = target

            target.modification = changelog
            target.save()



"""
Helper functions for the files in this dir

A small clarification for the order of event triggering:

-> request_started
-> pre_save
-> (pre_delete)
-> post_save
-> (post_delete)
-> m2m_changed
-> request_finished
"""

import logging
from django.contrib.auth.models import AnonymousUser
from .. import settings
from ..middleware import AutomatedLoggingMiddleware
from ..models import Application


def validate_instance(instance):
    """Validating if the instance should be logged, or is excluded"""
    excludes = settings.AUTOMATED_LOGGING['exclude']

    for excluded in excludes:
        if excluded in [instance._meta.app_label, instance.__class__.__name__] or instance.__module__.startswith(excluded):
            return False

    return True


def get_current_user():
    """Get current user object from middleware"""
    thread_local = AutomatedLoggingMiddleware.thread_local
    if hasattr(thread_local, 'current_user'):
        user = thread_local.current_user
        if isinstance(user, AnonymousUser):
            user = None
    else:
        user = None

    return user


def get_current_environ():
    """Get current application and path object from middleware"""
    thread_local = AutomatedLoggingMiddleware.thread_local
    if hasattr(thread_local, 'request_uri'):
        request_uri = thread_local.request_uri
    else:
        request_uri = None

    if hasattr(thread_local, 'application'):
        application = thread_local.application
        application = Application.objects.get_or_create(name=application)[0]
    else:
        application = None

    if hasattr(thread_local, 'method'):
        method = thread_local.method
    else:
        method = None

    if hasattr(thread_local, 'status'):
        status = thread_local.status
    else:
        status = None

    return request_uri, application, method, status


def processor(status, sender, instance, updated=None, addition=''):
    """
    This is the standard logging processor.

    This is used to send the log to the handler and to other systems.
    """
    logger = logging.getLogger(__name__)
    if validate_instance(instance):
        user = get_current_user()
        application = instance._meta.app_label
        model_name = instance.__class__.__name__
        level = settings.AUTOMATED_LOGGING['loglevel']['model']

        logger.log(level,
                   ('%s %s %s(%s) in %s%s' % (user, status, instance, model_name, application, addition)).replace("  ", " "),
                   extra={'action': 'model',
                          'data': {
                              'status': status,
                              'user': user,
                              'sender': sender,
                              'instance': instance,
                              'update_fields': updated
                          }
                          })



import threading
import django

if django.VERSION[0] == 1 and django.VERSION[1] < 10:
  from django.core.urlresolvers import resolve
else:
  from django.urls import resolve


class AutomatedLoggingMiddleware:
  thread_local = threading.local()

  def __init__(self, get_response=None):
    self.get_response = get_response

  def __call__(self, request):
    request_uri = request.get_full_path()

    if self.get_response is not None:
      response = self.get_response(request)

    AutomatedLoggingMiddleware.thread_local.current_user = request.user
    AutomatedLoggingMiddleware.thread_local.method = request.method
    AutomatedLoggingMiddleware.thread_local.request_uri = request_uri
    AutomatedLoggingMiddleware.thread_local.application = resolve(request.path).func.__module__.split('.')[0]

    if self.get_response is not None:
      AutomatedLoggingMiddleware.thread_local.status = response.status_cod

    return response



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2018-02-15 14:22
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='LDAP',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('action', models.TextField()),
                ('succeeded', models.NullBooleanField()),
                ('errorMessage', models.TextField(blank=True, null=True)),
                ('basedn', models.TextField(blank=True, null=True)),
                ('entry', models.TextField(blank=True, null=True)),
                ('objectClass', models.TextField(blank=True, null=True)),
                ('cn', models.TextField(blank=True, null=True)),
                ('existing_members', models.TextField(blank=True, null=True)),
                ('data_members', models.TextField(blank=True, null=True)),
                ('diff_members', models.TextField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'LDAP event log entry',
                'verbose_name_plural': 'LDAP event log entries',
            },
        ),
        migrations.CreateModel(
            name='Model',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('message', models.TextField(null=True)),
                ('action', models.PositiveSmallIntegerField(choices=[(0, 'n/a'), (1, 'add'), (2, 'change'), (3, 'delete')], default=0)),
                ('application', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='atl_model_application', to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name': 'model log entry',
                'verbose_name_plural': 'model log entries',
            },
        ),
        migrations.CreateModel(
            name='ModelChangelog',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'verbose_name': 'Model entry change',
                'verbose_name_plural': 'Model entry changes',
            },
        ),
        migrations.CreateModel(
            name='ModelModification',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ModelObject',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value', models.CharField(max_length=255)),
                ('model', models.CharField(max_length=255)),
                ('application', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='atl_modelobject_application', to='contenttypes.ContentType')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Request',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('url', models.URLField()),
                ('application', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'request event entry',
                'verbose_name_plural': 'request event entries',
            },
        ),
        migrations.CreateModel(
            name='Unspecified',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('message', models.TextField(null=True)),
                ('level', models.PositiveSmallIntegerField(default=20)),
                ('file', models.CharField(max_length=255, null=True)),
                ('line', models.PositiveIntegerField(null=True)),
            ],
            options={
                'verbose_name': ' logging entry (Errors, Warnings, Info)',
                'verbose_name_plural': ' logging entries (Errors, Warnings, Info)',
            },
        ),
        migrations.AddField(
            model_name='modelmodification',
            name='currently',
            field=models.ManyToManyField(related_name='changelog_current', to='automated_logging.ModelObject'),
        ),
        migrations.AddField(
            model_name='modelmodification',
            name='previously',
            field=models.ManyToManyField(related_name='changelog_previous', to='automated_logging.ModelObject'),
        ),
        migrations.AddField(
            model_name='modelchangelog',
            name='information',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='automated_logging.ModelObject'),
        ),
        migrations.AddField(
            model_name='modelchangelog',
            name='inserted',
            field=models.ManyToManyField(related_name='changelog_inserted', to='automated_logging.ModelObject'),
        ),
        migrations.AddField(
            model_name='modelchangelog',
            name='modification',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, to='automated_logging.ModelModification'),
        ),
        migrations.AddField(
            model_name='modelchangelog',
            name='removed',
            field=models.ManyToManyField(related_name='changelog_removed', to='automated_logging.ModelObject'),
        ),
        migrations.AddField(
            model_name='model',
            name='information',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='atl_model_information', to='automated_logging.ModelObject'),
        ),
        migrations.AddField(
            model_name='model',
            name='modification',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='automated_logging.ModelChangelog'),
        ),
        migrations.AddField(
            model_name='model',
            name='user',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]



import logging
from django.test import TestCase
from django.test import Client
from django.contrib.auth.models import User
from .models import *
from testmodels.models import Base, M2MTest, ForeignTest, OneToOneTest, OrdinaryTest
import string


class RequestTestCase(TestCase):
  def setUp(self):
    self.c = Client()

    self.user = User.objects.create_user(username='example', password='example')
    self.user.save()

  def test_request_based_database_logging_logged_in(self):
    self.c.login(username='example', password='example')
    response = self.c.get('')
    assert response.status_code == 200
    assert Request.objects.all().count() == 1

    entry = Request.objects.all()[0]
    assert entry.uri == '/'
    assert entry.user == self.user

    self.c.logout()

  def test_request_based_database_logging_logged_out(self):
    response = self.c.get('')
    assert response.status_code == 200
    assert Request.objects.all().count() == 1

    entry = Request.objects.all().order_by('-created_at')[0]
    assert entry.uri == '/'
    assert entry.user is None


# add database test, thx
class ModeltestCase(TestCase):
  def setUp(self):
    self.c = Client()

    self.user = User.objects.create_user(username='example', password='example')
    self.user.save()

    self.c.login(username='example', password='example')

    self.baseset = [Base() for _ in range(10)]
    [x.save() for x in self.baseset]

  def test_model_based_database_m2m(self):
    m2m = M2MTest()
    m2m.save()

    m2m.test.set(self.baseset[:-1])
    m2m.save()

    m2m.test.add(self.baseset[-1])
    m2m.save()

    m2m.test.remove(self.baseset[-1])
    m2m.save()

  def test_model_based_database_foreign(self):
    foreign = ForeignTest()
    foreign.save()

    foreign.test = self.baseset[2]
    foreign.save()

  def test_model_based_database_onetoone(self):
    one = OneToOneTest()
    one.save()

    one.test = self.baseset[1]
    one.save()

  def test_model_based_database_ordinary(self):
    ordinary = OrdinaryTest()
    ordinary.save()

    ordinary.test = string.ascii_lowercase
    ordinary.save()

    ordinary.delete()


class HandlerTestCase(TestCase):
  def setUp(self):
    self.logger = logging.getLogger(__name__)

  def test_handler_unspecified_catch(self):
    self.logger.error('TEST')
    assert Unspecified.objects.all().count() == 1
    assert Unspecified.objects.all()[0].message == 'TEST'



"""
This files handles processig all the request related signals.
These signals are all django interal ones.
"""
import logging
from logging import CRITICAL, WARNING

from .. import settings
from django.dispatch import receiver
from django.core.handlers.wsgi import WSGIRequest
from django.core.signals import request_finished, got_request_exception
from . import get_current_environ, get_current_user


@receiver(request_finished, weak=False)
def request_finished_callback(sender, **kwargs):
    """This function logs if the user acceses the page"""
    logger = logging.getLogger(__name__)
    level = settings.AUTOMATED_LOGGING['loglevel']['request']

    user = get_current_user()
    uri, application, method, status = get_current_environ()

    logger.log(level, ('%s performed request at %s (%s %s)' % (user, uri, method, status)).replace("  ", " "), extra={
        'action': 'request',
        'data': {
            'user': user,
            'uri': uri,
            'method': method,
            'application': application,
            'status': status
        }
    })


@receiver(got_request_exception, weak=False)
def request_exception(sender, request, **kwargs):
    """
    Automated request exception logging
    The function can also return an WSGIRequest exception,
    which does not supply either status_code or reason_phrase.
    """
    if not isinstance(request, WSGIRequest):
        logger = logging.getLogger(__name__)
        level = CRITICAL if request.status_code <= 500 else WARNING

        logger.log(level, '%s exception occured (%s)', request.status_code, request.reason_phrase)

    else:
        logger = logging.getLogger(__name__)
        logger.log(WARNING, 'WSGIResponse exception occured')



"""This file handles everything related to saving and deleting model objects."""
import re
import logging

from .. import settings
from django.db import transaction
from django.dispatch import receiver
from django.db.models.signals import post_delete
from django.db.models.signals import post_save, pre_save

from ..models import ModelChangelog, ModelModification, ModelObject, Field
from django.contrib.contenttypes.models import ContentType
from . import validate_instance, processor


@receiver(pre_save, weak=False)
def comparison_callback(sender, instance, **kwargs):
    """Comparing old and new object to determin which fields changed how"""
    if validate_instance(instance) and settings.AUTOMATED_LOGGING['to_database']:
        try:
            old = sender.objects.get(pk=instance.pk)
        except Exception:
            return None

        try:
            mdl = ContentType.objects.get_for_model(instance)
            cur, ins = old.__dict__, instance.__dict__
            old, new = {}, {}
            for k in cur.keys():
                # _ fields are not real model fields, only state or cache fields
                # getting filtered
                if re.match('(_)(.*?)', k):
                    continue

                changed = False
                if k in ins.keys() and cur[k] != ins[k]:
                    changed = True
                    new[k] = ModelObject()
                    new[k].value = str(ins[k])
                    new[k].save()

                    try:
                        new[k].type = ContentType.objects.get_for_model(ins[k])
                    except Exception:
                        logger = logging.getLogger(__name__)
                        logger.debug('Could not dermin the content type of the field')

                    new[k].field = Field.objects.get_or_create(name=k, model=mdl)[0]
                    new[k].save()
                else:
                    changed = True

                if changed:
                    old[k] = ModelObject()
                    old[k].value = str(cur[k])
                    old[k].save()

                    try:
                        old[k].type = ContentType.objects.get_for_model(cur[k])
                    except Exception:
                        logger = logging.getLogger(__name__)
                        logger.debug('Could not dermin the content type of the field')

                    old[k].field = Field.objects.get_or_create(name=k, model=mdl)[0]
                    old[k].save()

            if old:
                changelog = ModelChangelog()
                changelog.save()

                changelog.modification = ModelModification()
                changelog.modification.save()
                changelog.modification.previously.set(old.values())
                changelog.modification.currently.set(new.values())

                changelog.information = ModelObject()
                changelog.information.save()
                changelog.information.value = repr(instance)
                changelog.information.type = ContentType.objects.get_for_model(instance)
                changelog.information.save()
                changelog.save()

                instance.al_chl = changelog

                return instance

        except Exception as e:
            print(e)
            logger = logging.getLogger(__name__)
            logger.warning('automated_logging recorded an exception that should not have happended')


@receiver(post_save, weak=False)
def save_callback(sender, instance, created, update_fields, **kwargs):
    """Save object & link logging entry"""
    if validate_instance(instance):
        status = 'add' if created is True else 'change'
        change = ''

        if status == 'modified' and 'al_chl' in instance.__dict__.keys():
            changelog = instance.al_chl.modification
            change = ' and changed {0} to {1}'.format(changelog.previously, changelog.currently)

        processor(status, sender, instance, update_fields, addition=change)


@receiver(post_delete, weak=False)
@transaction.atomic
def delete_callback(sender, instance, **kwargs):
    """Triggered when deleted -> logged"""
    status = 'delete'
    processor(status, sender, instance)



from django.apps import AppConfig
from .settings import AUTOMATED_LOGGING


class AutomatedloggingConfig(AppConfig):
    name = 'automated_logging'
    verbose_name = 'Django Automated Logging (DAL)'

    def ready(self):
      if 'request' in AUTOMATED_LOGGING['modules']:
        from .signals import request
      if 'model' in AUTOMATED_LOGGING['modules']:
        from .signals import database, m2m

      from .handlers import DatabaseHandler



# Generated by Django 2.0.2 on 2018-02-18 11:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('automated_logging', '0011_auto_20180216_1545'),
    ]

    operations = [
        migrations.AddField(
            model_name='request',
            name='method',
            field=models.CharField(default='GET', max_length=64),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='request',
            name='url',
            field=models.CharField(max_length=255),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2018-02-16 09:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('automated_logging', '0002_auto_20180215_1540'),
    ]

    operations = [
        migrations.AlterField(
            model_name='modelobject',
            name='value',
            field=models.CharField(max_length=255, null=True),
        ),
    ]



# Generated by Django 2.0.2 on 2018-02-18 11:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('automated_logging', '0012_auto_20180218_1101'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='request',
            name='url',
        ),
        migrations.AddField(
            model_name='request',
            name='uri',
            field=models.URLField(default='/'),
            preserve_default=False,
        ),
    ]



"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""
from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='django-automated-logging',
    version='3.3.5',
    description='Django Database Based Automated Logging - finally solved and done in a proper way.',
    long_description=long_description,
    url='https://github.com/indietyp/django-automated-logging',
    author='Bilal Mahmoud',
    author_email='opensource@indietyp.com',
    license='MIT',
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    include_package_data=True,
    keywords='django automation tools backend',
    packages=find_packages(exclude=(['logtest', 'test*'])),
    install_requires=['Django>=1.10'],
    zip_safe=False
)



from django.contrib import admin
from . import models


class ReadOnlyAdminMixin(admin.ModelAdmin):
    """Disables all editing capabilities."""

    change_form_template = "admin/view.html"

    def __init__(self, *args, **kwargs):
        super(ReadOnlyAdminMixin, self).__init__(*args, **kwargs)
        self.readonly_fields = [f.name for f in self.model._meta.get_fields()]

    def get_actions(self, request):
        actions = super(ReadOnlyAdminMixin, self).get_actions(request)
        del actions["delete_selected"]
        return actions

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def save_model(self, request, obj, form, change):
        pass

    def delete_model(self, request, obj):
        pass

    def save_related(self, request, form, formsets, change):
        pass


class ModelAdmin(ReadOnlyAdminMixin):
    def get_prev(self, obj):
        if obj.modification is not None and obj.modification.modification is not None:
            return ", ".join(str(v) for v in obj.modification.modification.previously.all())
        else:
            return ''
    get_prev.short_description = 'Previous Value'

    def get_curr(self, obj):
        if obj.modification is not None and obj.modification.modification is not None:
            return ", ".join(str(v) for v in obj.modification.modification.currently.all())
        else:
            return ''
    get_curr.short_description = 'Changed Value'

    def get_remo(self, obj):
        if obj.modification is not None and obj.modification.removed is not None:
            return ", ".join(str(v) for v in obj.modification.removed.all())
        else:
            return ''
    get_remo.short_description = 'Removed'

    def get_inser(self, obj):
        if obj.modification is not None and obj.modification.inserted is not None:
            return ", ".join(str(v) for v in obj.modification.inserted.all())
        else:
            return ''
    get_inser.short_description = 'Inserted'

    def get_infor(self, obj):
        if obj.information is not None and obj.information.type is not None:
            return obj.information.type.model + '.' + str(obj.application)
        else:
            return ''
    get_infor.short_description = 'Object'

    list_display = (
        'user',
        'action',
        'get_prev',
        'get_curr',
        'get_remo',
        'get_inser',
        'get_infor',
        'updated_at',
    )

    list_filter = (
        'updated_at',
        'user',
    )

    date_hierarchy = 'updated_at'
    ordering = ('-updated_at',)


class RequestAdmin(ReadOnlyAdminMixin):

    list_display = (
        'user',
        'method',
        'uri',
        'application',
        'updated_at'
    )
    list_filter = ('updated_at', 'application', 'user')
    date_hierarchy = 'updated_at'
    ordering = ('-updated_at',)


class UnspecifiedAdmin(ReadOnlyAdminMixin):

    list_display = (
        'level',
        'file',
        'line',
        'message',
        'updated_at'
    )
    list_filter = ('created_at', 'updated_at')
    date_hierarchy = 'created_at'
    ordering = ('-updated_at',)


# class LDAPAdmin(ReadOnlyAdminMixin):

#     list_display = (
#         'action',
#         'succeeded',
#         'errorMessage',
#         'basedn',
#         'entry',
#         'objectClass',
#         'cn',
#         'existing_members',
#         'data_members',
#         'diff_members',
#     )
#     list_filter = ('created_at', 'updated_at')
#     date_hierarchy = 'created_at'


def _register(model, admin_class):
    admin.site.register(model, admin_class)


_register(models.Model, ModelAdmin)
_register(models.Request, RequestAdmin)
_register(models.Unspecified, UnspecifiedAdmin)
# _register(models.LDAP, LDAPAdmin)



from django.conf import settings as st
from logging import INFO

DEFAULT_AUTOMATED_LOGGING = {
    'exclude': ['Session', 'automated_logging', 'basehttp'],
    'modules': ['request', 'model', 'unspecified'],
    'to_database': True,
    'loglevel': {'model': INFO,
                 'request': INFO}
}


def auto_complete(setting, default):
  for k, v in default.items():
    if k not in setting.keys():
      setting[k] = v
    elif isinstance(setting[k], dict):
      setting[k] = auto_complete(setting[k], default[k])

  return setting


if hasattr(st, 'AUTOMATED_LOGGING'):
  AUTOMATED_LOGGING = auto_complete(st.AUTOMATED_LOGGING, DEFAULT_AUTOMATED_LOGGING)
else:
  AUTOMATED_LOGGING = DEFAULT_AUTOMATED_LOGGING



"""
This file contains the custom database based django ORM handler. This is just a bit hacky.
Some might even say this is just sorcery and magic.

Hopefully it is not.
"""
from logging import Handler


class DatabaseHandler(Handler):
    """Handler for logging into any database"""

    def __init__(self):
        super(DatabaseHandler, self).__init__()

    def emit(self, record):
        """
        add to database
        try - except -> preventing circular import
        http://stackoverflow.com/questions/4379042/django-circular-model-import-issue
        """

        try:
            from .models import Model, Application, ModelObject
            from django.contrib.contenttypes.models import ContentType

            if not hasattr(record, 'action'):
                from .settings import AUTOMATED_LOGGING

                signal = True
                for excluded in AUTOMATED_LOGGING['exclude']:
                    if record.module.startswith(excluded):
                        signal = False
                        break

                if 'unspecified' not in AUTOMATED_LOGGING['modules']:
                    signal = False

                if signal:
                    from .models import Unspecified

                    entry = Unspecified()

                    if hasattr(record, 'message'):
                        entry.message = record.message

                    entry.level = record.levelno
                    entry.file = record.pathname
                    entry.line = record.lineno

                    entry.save()

                return

            if record.action == 'model':
                if 'al_evt' in record.data['instance'].__dict__.keys():
                    entry = record.data['instance'].al_evt
                else:
                    entry = Model()
                    entry.user = record.data['user']
                    entry.application = Application.objects.get_or_create(name=record.data['instance']._meta.app_label)[0]

                    entry.message = record.message

                    if record.data['status'] == 'add':
                        status = 1
                    elif record.data['status'] == 'change':
                        status = 2
                    elif record.data['status'] == 'delete':
                        status = 3
                    else:
                        status = 0

                    entry.action = status
                    entry.information = ModelObject()
                    entry.information.value = repr(record.data['instance'])
                    ct = ContentType.objects.get_for_model(record.data['instance'])

                    try:
                        # check if the ContentType actually exists.
                        ContentType.objects.get(pk=ct.pk)
                        ct_exists = True
                    except ContentType.DoesNotExist:
                        ct_exists = False

                    if ct_exists:
                        entry.information.type = ct

                    entry.information.save()

                    record.data['instance'].al_evt = entry

                if record.data['status'] == 'modified' and 'al_chl' in record.data['instance'].__dict__.keys():
                    entry.modification = record.data['instance'].al_chl

                entry.save()

            elif record.action == 'request':
                from .models import Request

                entry = Request()
                entry.application = record.data['application']
                entry.uri = record.data['uri']
                entry.user = record.data['user']
                entry.status = record.data['status']
                entry.method = record.data['method']
                entry.save()

        except Exception as e:
            print(e)



"""Database table definitions for the application, everything is logging related."""
from django.conf import settings
import uuid
from django.db import models
from django.contrib.contenttypes.models import ContentType


class BaseModel(models.Model):
    """BaseModel that is inherited from every model. Includes basic information."""

    id = models.UUIDField(default=uuid.uuid4, primary_key=True)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class Application(BaseModel):
    """
    Table for every application that might be used
    - this is not created get_or_create,
    so it is not yet a full representation of every application installed,
    this might follow
    """

    name = models.CharField(max_length=255)

    def __str__(self):
        """Returns name of application."""
        return self.name


class Field(BaseModel):
    """
    Table definition for a regular field. Is tied to a ContentType
    If the model will be deleted all the related fields will be therefor too.
    """

    name = models.CharField(max_length=255)
    model = models.ForeignKey(ContentType, null=True, on_delete=models.CASCADE, related_name='dal_field')

    def __str__(self):
        return '{} - {}'.format(self.name, self.model)


class ModelObject(BaseModel):
    """
    BaseObject for everything logging related.
    consists of a value: gathered through repr()
    field - which is a definition of the field
    and if it refers to a relationship the model.

    BaseObject of system.
    """

    value = models.CharField(max_length=255, null=True)
    field = models.ForeignKey(Field, null=True, on_delete=models.CASCADE)
    type = models.ForeignKey(ContentType, null=True, blank=True, on_delete=models.CASCADE, related_name='atl_modelobject_application')

    def __str__(self):
        output = '{}'.format(self.value)

        if self.field is not None:
            output += " of field {}".format(self.field)

        if self.type is not None:
            output += " in model {}.{}".format(self.type.app_label, self.type.model)

        return output


class ModelModification(BaseModel):
    """Saves the two states of several fields."""

    previously = models.ManyToManyField(ModelObject, related_name='changelog_previous')
    currently = models.ManyToManyField(ModelObject, related_name='changelog_current')

    def __str__(self):
        print([str(v) for v in self.previously.all()])
        return ' {0} changed to {1}; '.format(", ".join(str(v) for v in self.previously.all()),
                                              ", ".join(str(v) for v in self.currently.all()))


class ModelChangelog(BaseModel):
    """General changelog, saves which fields are removede, inserted (both m2m) and which are modified."""

    modification = models.OneToOneField(ModelModification, null=True, on_delete=models.CASCADE)
    inserted = models.ManyToManyField(ModelObject, related_name='changelog_inserted')
    removed = models.ManyToManyField(ModelObject, related_name='changelog_removed')

    information = models.OneToOneField(ModelObject, null=True, on_delete=models.CASCADE)

    def __str__(self):
        output = ''
        if self.modification is not None:
            output += '{0} in {1}'.format(self.modification, self.information)
        if self.inserted.count() > 0:
            output += '{0} was inserted into {1}; '.format(", ".join(str(v) for v in self.inserted.all()), self.information)
        if self.removed.count() > 0:
            output += '{0} was removed from {1}; '.format(", ".join(str(v) for v in self.removed.all()), self.information)

        return output


class Model(BaseModel):
    """This ties a changelog to a specific user, this is used by the DatabaseHandler."""

    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, on_delete=models.CASCADE)
    application = models.ForeignKey(Application, on_delete=models.CASCADE, related_name='atl_model_application', null=True)

    message = models.TextField(null=True)

    MODES = (
        (0, 'n/a'),
        (1, 'add'),
        (2, 'change'),
        (3, 'delete'),
    )
    action = models.PositiveSmallIntegerField(choices=MODES, default=0)

    information = models.OneToOneField(ModelObject, on_delete=models.CASCADE, related_name='atl_model_information', null=True)
    modification = models.ForeignKey(ModelChangelog, null=True, on_delete=models.CASCADE)

    def __str__(self):
        return '{0} - {1}'.format(self.created_at, self.message)

    class Meta:
        verbose_name = 'Changelog'
        verbose_name_plural = 'Changelogs'


class Request(BaseModel):
    """The model where every request is saved."""

    application = models.ForeignKey(Application, on_delete=models.CASCADE, null=True)

    uri = models.URLField()
    method = models.CharField(max_length=64)
    status = models.PositiveSmallIntegerField(null=True)

    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, on_delete=models.CASCADE)

    def __str__(self):
        return '{2} - {0} performed request at {3} ({1})'.format(self.user, self.created_at, self.uri, self.application)

    class Meta:
        verbose_name = "Request"
        verbose_name_plural = "Requests"


class Unspecified(BaseModel):
    """Logging messages that are saved by non DAL systems."""

    message = models.TextField(null=True)
    level = models.PositiveSmallIntegerField(default=20)

    file = models.CharField(max_length=255, null=True)
    line = models.PositiveIntegerField(null=True)

    def __str__(self):
        if self.level == 10:
            level = 'DEBUG'
        elif self.level == 20:
            level = 'INFO'
        elif self.level == 30:
            level = 'WARNING'
        elif self.level == 40:
            level = 'ERROR'
        elif self.level == 50:
            level = 'CRITICAL'
        else:
            level = 'NOTSET'

        return '{} - {} - {} ({} - {})'.format(self.created_at, level, self.message, self.file, self.line)

    class Meta:
        verbose_name = "Non DJL Message"
        verbose_name_plural = "Non DJL Messages"


class LDAP(BaseModel):
    """
    LDAP model definition, not used by DAL
    IS USED IN OTHER SYSTEMS.
    """

    action = models.TextField()
    succeeded = models.NullBooleanField(blank=True, null=True)
    errorMessage = models.TextField(blank=True, null=True)

    basedn = models.TextField(blank=True, null=True)
    entry = models.TextField(blank=True, null=True)

    objectClass = models.TextField(blank=True, null=True)
    cn = models.TextField(blank=True, null=True)

    existing_members = models.TextField(blank=True, null=True)
    data_members = models.TextField(blank=True, null=True)
    diff_members = models.TextField(blank=True, null=True)

    class Meta:
        verbose_name = "LDAP event log entry"
        verbose_name_plural = "LDAP event log entries"
